/*
   _===_ _           _                 ______ _     _             _ _ _
  |  ___| |         | |                | ___ (_)   | |           (_) | |
  | |__ | | ___  ___| |_ _ __ ___ _____| |_/ /_  __| | ___  _   _ _| | | ___ _   _ _ __
  |  __|| |/ _ \/ __| __| '__/ _ \_____| ___ \ |/ _` |/ _ \| | | | | | |/ _ \ | | | '__|
  | |___| |  __/ (__| |_| | | (_) |    | |_/ / | (_| | (_) | |_| | | | |  __/ |_| | |
  \____/|_|\___|\___|\__|_|  \___/     \____/|_|\__,_|\___/ \__,_|_|_|_|\___|\__,_|_|

Contrôle/Configuration du Synthétiseur Si5351 pour l'émetteur AM
Conçu pour rouler sur un Arduino-Nano 
Électro-Bidouilleur - Septembre 2021
Licence de droits: Attribution 4.0 International (CC BY 4.0)  
  Lisez https://creativecommons.org/licenses/by/4.0/deed.fr  

La librairie si5351 de Jason Milldrum est requise: 
 * Copyright (C) 2015 - 2016 Jason Milldrum <milldrum@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "si5351.h"
#include "Wire.h"

// Constantes
#define PLL_LED 11    // La LED de statut du PLL du Si5351

Si5351 si5351;

void setup()
{
  bool i2c_found;

  // Initialisation du port série 
    Serial.begin(9600);

  //Initialisation de la broche de LED
  pinMode(PLL_LED, OUTPUT);     // Définition de la broche de la LED de statut du PLL

  //Initialisation du synthétiseur Si5351
  i2c_found = si5351.init(SI5351_CRYSTAL_LOAD_8PF, 0, 0); 
  si5351.set_pll_input(SI5351_PLLA, SI5351_PLL_INPUT_XO);
  if(!i2c_found)
  {
    Serial.println("Synthétiseur non détecté sur le bus I2C!");
  }

  // Établissement de la fréquence de sortie CLK0
  // Saisie de la fréquence en centièmes de Hz. Exemple: 1000 KHz est 100000000. 
  // Il faut aussi annexer les caractères "ULL" à  la fréquence.
  si5351.set_freq(167000000ULL, SI5351_CLK0); 
  
  // Query a status update and wait a bit to let the Si5351 populate the
  // status flags correctly.
  //  si5351.update_status();
  // delay(500);
}

void loop()
{
bool pll_state = 0;
static unsigned long timer = millis();

  // Détection du signal référence externe pour activer la LED. Lecture directe du registre de statut. 
  // Anti-rebond nécessaire.
  pll_state = !((si5351.si5351_read(0) & 0b00100000)>>5);  // Le statut LOL est plus stable que le LOS  
  if (pll_state)    // Le PLL est verrouillé
  {
    digitalWrite(PLL_LED,1);
    timer = millis();
  }
  else if (millis() - timer > 1000)  // Le PLL est déverrouillé et le temps de 1 seconde est écoulé
  {
    digitalWrite(PLL_LED,0);  // Le statut LOL est plus stable que le LOS
    timer = millis();
  }
}
